#include <sourcemod>
#include <sdktools>
#include <sdkhooks>

#pragma semicolon 1

#define PLUGIN_VERSION "2.0"

new Handle:hDamage;
new Handle:hRadius;
new Handle:hSpeed;
new Handle:hType;
new Handle:hEnable;
new Handle:hTeam;

new Float:MinNadeHull[3] = {-2.5, -2.5, -2.5};
new Float:MaxNadeHull[3] = {2.5, 2.5, 2.5};
new Float:MaxWorldLength;
new Float:SpinVel[3] = {0.0, 0.0, 200.0};
new Float:SmokeOrigin[3] = {-30.0,0.0,0.0};
new Float:SmokeAngle[3] = {0.0,-180.0,0.0};

new NadeDamage;
new NadeRadius;
new Float:NadeSpeed;
new NadeType;
new Float:NadeArc;
new NadeAllowTeam;
new IsEnabled;

new String:ReplaceNade[][] =
{
	"grenade_frag_us",
	"grenade_frag_ger",
	"grenade_smoke_us",
	"grenade_smoke_ger"
};

public Plugin:myinfo =
{
	name = "Dod Homing Missiles",
	author = "Andi67",
	description = "Turns projectile weapons into homing missiles",
	version = "2.0",
	url = ""
}

public OnPluginStart()
{
	CreateConVar("dod_missile_version", PLUGIN_VERSION, "Dod Homing missile version", FCVAR_PLUGIN|FCVAR_NOTIFY|FCVAR_REPLICATED);
	hDamage = CreateConVar("dod_missile_damage", "100", "Sets the maximum amount of damage the missiles can do", FCVAR_PLUGIN, true, 1.0);
	hRadius = CreateConVar("dod_missile_radius", "350", "Sets the explosive radius of the missiles", FCVAR_PLUGIN, true, 1.0);
	hSpeed = CreateConVar("dod_missile_speed", "500.0", "Sets the speed of the missiles", FCVAR_PLUGIN, true, 300.0 ,true, 3000.0);
	hType = CreateConVar("dod_missile_type", "1", "type of missile to use, 0 = dumb missiles, 1 = homing missiles, 2 = crosshair guided", FCVAR_PLUGIN, true, 0.0, true, 2.0);
	hEnable = CreateConVar("dod_missile_enable", "1", "1 enables plugin, 0 disables plugin", FCVAR_PLUGIN, true, 0.0, true, 1.0);
	hTeam = CreateConVar("dod_missile_team", "0", "which team can use missiles, 0 = any, 1 = only terrorists, 2 = only counter terrorists", FCVAR_PLUGIN, true, 0.0, true, 2.0);
	
	NadeDamage = GetConVarInt(hDamage);
	NadeRadius = GetConVarInt(hRadius);
	NadeSpeed = GetConVarFloat(hSpeed);
	NadeType = GetConVarInt(hType);
	NadeAllowTeam = GetConVarInt(hTeam) + 1;
	NadeArc = NadeSpeed / 1000.0;
	
	if (GetConVarInt(hEnable))
	{
		IsEnabled = 1;
	}
	
	HookConVarChange(hDamage, ConVarChange);
	HookConVarChange(hRadius, ConVarChange);
	HookConVarChange(hSpeed, ConVarChange);
	HookConVarChange(hType, ConVarChange);
	HookConVarChange(hEnable, ConVarChange);
	HookConVarChange(hTeam, ConVarChange);
}

public OnMapStart()
{
	new Float:WorldMinHull[3], Float:WorldMaxHull[3];
	GetEntPropVector(0, Prop_Send, "m_WorldMins", WorldMinHull);
	GetEntPropVector(0, Prop_Send, "m_WorldMaxs", WorldMaxHull);
	MaxWorldLength = GetVectorDistance(WorldMinHull, WorldMaxHull);
	
	AddFileToDownloadsTable("sound/weapons/rpg/rocket1.wav");
	
	AddFileToDownloadsTable("materials/models/weapons/w_missile/missile side.vmt");
	
	AddFileToDownloadsTable("models/weapons/W_missile_closed.dx80.vtx");
	AddFileToDownloadsTable("models/weapons/W_missile_closed.dx90.vtx");
	AddFileToDownloadsTable("models/weapons/W_missile_closed.mdl");
	AddFileToDownloadsTable("models/weapons/W_missile_closed.phy");
	AddFileToDownloadsTable("models/weapons/W_missile_closed.sw.vtx");
	AddFileToDownloadsTable("models/weapons/W_missile_closed.vvd");
	
	PrecacheModel("models/weapons/w_missile_closed.mdl");
	
	PrecacheSound("weapons/rpg/rocket1.wav");
	PrecacheSound("weapons/explode5.wav");
}

public ConVarChange(Handle:cvar, const String:oldVal[], const String:newVal[])
{
	if (cvar == hDamage)
	{
		NadeDamage = StringToInt(newVal);
	}
	
	else if (cvar == hRadius)
	{
		NadeRadius = StringToInt(newVal);
	}
	
	else if (cvar == hSpeed)
	{
		NadeSpeed = StringToFloat(newVal);
		NadeArc = NadeSpeed / 1000.0;
	}
	
	else if (cvar == hType)
	{
		NadeType = StringToInt(newVal);
	}
	
	else if (cvar == hEnable)
	{
		IsEnabled = StringToInt(newVal);
	}
	
	else if (cvar == hTeam)
	{
		NadeAllowTeam = GetConVarInt(hTeam) + 1;
	}
}

public OnEntityCreated(entity, const String:classname[])
{
	for (new i = 0; i < sizeof(ReplaceNade); i++)
	{
		if (StrEqual(classname, ReplaceNade[i]))
		{
			if (!IsEnabled)
			{
				return;
			}
		
			HookSingleEntityOutput(entity, "OnUser2", InitMissile, true);
			
			decl String:OutputString[50] = "OnUser1 !self:FireUser2::0.01:1";
			SetVariantString(OutputString);
			AcceptEntityInput(entity, "AddOutput");
		
			AcceptEntityInput(entity, "FireUser1");
		}
	}
}

public InitMissile(const String:output[], caller, activator, Float:delay)
{
	new NadeTeam = GetEntProp(caller, Prop_Send, "m_iTeamNum");
	if ((NadeAllowTeam != 1) && (NadeAllowTeam != NadeTeam))
	{
		return;
	}
	
	SetEntProp(caller, Prop_Data, "m_nNextThinkTick", -1);
	SetEntityMoveType(caller, MOVETYPE_FLY);
	SetEntityModel(caller, "models/weapons/w_missile_closed.mdl");
	SetEntPropVector(caller, Prop_Data, "m_vecAngVelocity", SpinVel);
	SetEntPropVector(caller, Prop_Send, "m_vecMins", MinNadeHull);
	SetEntPropVector(caller, Prop_Send, "m_vecMaxs", MaxNadeHull);
	
	switch (NadeTeam)
	{
		case 2:
		{
			SetEntityRenderColor(caller, 0, 255, 80, 150);
		}
		case 3:
		{
			SetEntityRenderColor(caller, 255, 25, 25, 150);
		}
	}
	
	new SmokeIndex = CreateEntityByName("env_rockettrail");
	if (SmokeIndex != -1)
	{
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_Opacity", 0.5);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_SpawnRate", 100.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_ParticleLifetime", 0.5);
		new Float:SmokeRed[3] = {0.5, 0.25, 0.25};
		new Float:SmokeBlue[3] = {0.25, 0.25, 0.5};
		switch (NadeTeam)
		{
			case 2:
			{
				SetEntPropVector(SmokeIndex, Prop_Send, "m_StartColor", SmokeRed);
			}
			case 3:
			{
				SetEntPropVector(SmokeIndex, Prop_Send, "m_StartColor", SmokeBlue);
			}
		}
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_StartSize", 5.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_EndSize", 30.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_SpawnRadius", 0.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_MinSpeed", 0.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_MaxSpeed", 10.0);
		SetEntPropFloat(SmokeIndex, Prop_Send, "m_flFlareScale", 1.0);
		
		DispatchSpawn(SmokeIndex);
		ActivateEntity(SmokeIndex);
		
		new String:NadeName[20];
		Format(NadeName, sizeof(NadeName), "Nade_%i", caller);
		DispatchKeyValue(caller, "targetname", NadeName);
		SetVariantString(NadeName);
		AcceptEntityInput(SmokeIndex, "SetParent");
		TeleportEntity(SmokeIndex, SmokeOrigin, SmokeAngle, NULL_VECTOR);
	}
	
	decl Float:NadePos[3];
	GetEntPropVector(caller, Prop_Send, "m_vecOrigin", NadePos);
	new NadeOwner = GetEntPropEnt(caller, Prop_Send, "m_hThrower");
	decl Float:OwnerAng[3];
	GetClientEyeAngles(NadeOwner, OwnerAng);
	decl Float:OwnerPos[3];
	GetClientEyePosition(NadeOwner, OwnerPos);
	TR_TraceRayFilter(OwnerPos, OwnerAng, MASK_SOLID, RayType_Infinite,  RayDontHitOwnerOrNade, caller);
	decl Float:InitialPos[3];
	TR_GetEndPosition(InitialPos);
	decl Float:InitialVec[3];
	MakeVectorFromPoints(NadePos, InitialPos, InitialVec);
	NormalizeVector(InitialVec, InitialVec);
	ScaleVector(InitialVec, NadeSpeed);
	decl Float:InitialAng[3];
	GetVectorAngles(InitialVec, InitialAng);
	TeleportEntity(caller, NULL_VECTOR, InitialAng, InitialVec);
	
	EmitSoundToAll("weapons/rpg/rocket1.wav", caller, 1, 90);
	
	HookSingleEntityOutput(caller, "OnUser2", MissileThink);
	
	decl String:OutputString[50] = "OnUser1 !self:FireUser2::0.1:-1";
	SetVariantString(OutputString);
	AcceptEntityInput(caller, "AddOutput");
	
	AcceptEntityInput(caller, "FireUser1");
	
	SDKHook(caller, SDKHook_StartTouch, OnStartTouch);
}

public MissileThink(const String:output[], caller, activator, Float:delay)
{
	decl Float:CheckVec[3];
	GetEntPropVector(caller, Prop_Send, "m_vecVelocity", CheckVec);
	if ((CheckVec[0] == 0.0) && (CheckVec[1] == 0.0) && (CheckVec[2] == 0.0))
	{
		StopSound(caller, 1, "weapons/rpg/rocket1.wav");
		CreateExplosion(caller);
		return;
	}
	
	decl Float:NadePos[3];
	GetEntPropVector(caller, Prop_Send, "m_vecOrigin", NadePos);
	
	new NadeTeam = GetEntProp(caller, Prop_Send, "m_iTeamNum");
	
	if (NadeType > 0)
	{
		new Float:ClosestDistance = MaxWorldLength;
		decl Float:TargetVec[3];
		
		if (NadeType == 1)
		{
			new ClosestEnemy;
			new Float:EnemyDistance;
			for (new i = 1; i <= MaxClients; i++)
			{
				if (IsClientInGame(i) && IsPlayerAlive(i) && (GetClientTeam(i) != NadeTeam))
				{
					decl Float:EnemyPos[3];
					GetClientEyePosition(i, EnemyPos);
					TR_TraceHullFilter(NadePos, EnemyPos, MinNadeHull, MaxNadeHull, MASK_SOLID, RayDontHitOwnerOrNade, caller);
					if (TR_GetEntityIndex() == i)
					{
						EnemyDistance = GetVectorDistance(NadePos, EnemyPos);
						if (EnemyDistance < ClosestDistance)
						{
							ClosestEnemy = i;
							ClosestDistance = EnemyDistance;
						}
					}
				}
			}
			if (ClosestEnemy == 0)
			{
				AcceptEntityInput(caller, "FireUser1");
				return;
			}
			else
			{
				decl Float:EnemyPos[3];
				GetClientEyePosition(ClosestEnemy, EnemyPos);
				MakeVectorFromPoints(NadePos, EnemyPos, TargetVec);
			}
		}
		
		else if (NadeType == 2)
		{
			new NadeOwner = GetEntPropEnt(caller, Prop_Send, "m_hThrower");
			decl Float:OwnerAng[3];
			GetClientEyeAngles(NadeOwner, OwnerAng);
			decl Float:OwnerPos[3];
			GetClientEyePosition(NadeOwner, OwnerPos);
			TR_TraceRayFilter(OwnerPos, OwnerAng, MASK_SOLID, RayType_Infinite,  RayDontHitOwnerOrNade, caller);
			decl Float:TargetPos[3];
			TR_GetEndPosition(TargetPos);
			ClosestDistance = GetVectorDistance(NadePos, TargetPos);
			MakeVectorFromPoints(NadePos, TargetPos, TargetVec);
		}
		
		decl Float:CurrentVec[3];
		GetEntPropVector(caller, Prop_Send, "m_vecVelocity", CurrentVec);
		decl Float:FinalVec[3];
		if (ClosestDistance > 100.0)
		{
			NormalizeVector(TargetVec, TargetVec);
			NormalizeVector(CurrentVec, CurrentVec);
			ScaleVector(TargetVec, NadeArc);
			AddVectors(TargetVec, CurrentVec, FinalVec);
		}
		else
		{
			FinalVec = TargetVec;
		}
		NormalizeVector(FinalVec, FinalVec);
		ScaleVector(FinalVec, NadeSpeed);
		decl Float:FinalAng[3];
		GetVectorAngles(FinalVec, FinalAng);
		TeleportEntity(caller, NULL_VECTOR, FinalAng, FinalVec);
	}
	
	AcceptEntityInput(caller, "FireUser1");
}

public bool:RayDontHitOwnerOrNade(entity, contentsMask, any:data)
{
	new NadeOwner = GetEntPropEnt(data, Prop_Send, "m_hThrower");
	return ((entity != data) && (entity != NadeOwner));
}

public OnStartTouch(entity, other) 
{
	if ((GetEntProp(other, Prop_Send, "m_nSolidType") != 0) && ((GetEntProp(other, Prop_Send, "m_usSolidFlags") & 8) == 0))
	{
		StopSound(entity, 1, "weapons/rpg/rocket1.wav");
		CreateExplosion(entity);
	}
}

public CreateExplosion(entity)
{
	UnhookSingleEntityOutput(entity, "OnUser2", MissileThink);
	
	new Float:MissilePos[3];
	GetEntPropVector(entity, Prop_Send, "m_vecOrigin", MissilePos);
	new MissileOwner = GetEntPropEnt(entity, Prop_Send, "m_hThrower");
	new MissileOwnerTeam = GetEntProp(entity, Prop_Send, "m_iTeamNum");
	
	new ExplosionIndex = CreateEntityByName("env_explosion");
	if (ExplosionIndex != -1)
	{
		SetEntProp(ExplosionIndex, Prop_Data, "m_spawnflags", 6144);
		SetEntProp(ExplosionIndex, Prop_Data, "m_iMagnitude", NadeDamage);
		SetEntProp(ExplosionIndex, Prop_Data, "m_iRadiusOverride", NadeRadius);
		
		DispatchSpawn(ExplosionIndex);
		ActivateEntity(ExplosionIndex);
		
		TeleportEntity(ExplosionIndex, MissilePos, NULL_VECTOR, NULL_VECTOR);
		SetEntPropEnt(ExplosionIndex, Prop_Send, "m_hOwnerEntity", MissileOwner);
		SetEntProp(ExplosionIndex, Prop_Send, "m_iTeamNum", MissileOwnerTeam);
		
		EmitSoundToAll("weapons/explode5.wav", ExplosionIndex, 1, 90);
		
		AcceptEntityInput(ExplosionIndex, "Explode");
	}
	
	AcceptEntityInput(entity, "Kill");
}